searchData={"items":[{"type":"module","title":"Cucumber","doc":"A behavior-driven development (BDD) testing framework for Elixir using Gherkin syntax.\n\nCucumber is a testing framework that allows you to write executable specifications\nin natural language. It bridges the gap between technical and non-technical stakeholders\nby allowing tests to be written in plain language while being executed as code.","ref":"Cucumber.html"},{"type":"module","title":"Usage - Cucumber","doc":"To use Cucumber in your test file:\n\n    defmodule UserAuthenticationTest do\n      use Cucumber, feature: \"user_authentication.feature\"\n      \n      defstep \"I am on the sign in page\", context do\n        # Step implementation\n        Map.put(context, :current_page, :sign_in)\n      end\n      \n      # More step definitions\n    end\n\nYou can also filter scenarios by tags:\n\n    # Only run scenarios tagged with \"smoke\" or \"auth\"\n    use Cucumber, feature: \"user_authentication.feature\", tags: [\"smoke\", \"auth\"]","ref":"Cucumber.html#module-usage"},{"type":"module","title":"Key Features - Cucumber","doc":"* Gherkin Support - Write tests in familiar Given/When/Then format\n* Parameter Types - Define step patterns with typed parameters like `{string}`, `{int}`\n* Data Tables - Pass structured data to your steps\n* DocStrings - Include multi-line text blocks in your steps\n* Background Steps - Define common setup steps for all scenarios\n* Tag Filtering - Run subsets of scenarios using tags\n* Context Passing - Share state between steps with a simple context map\n* Rich Error Reporting - Clear error messages with step execution history","ref":"Cucumber.html#module-key-features"},{"type":"function","title":"Cucumber.apply_step/3","doc":"Applies a step from a feature file to a matching step definition.\n\nThis function is used internally by the Cucumber framework to execute steps.\nIt handles the pattern matching, parameter extraction, and context management.","ref":"Cucumber.html#apply_step/3"},{"type":"function","title":"Parameters - Cucumber.apply_step/3","doc":"- `module` - The test module containing step definitions\n- `context` - The current context map\n- `step` - The `Gherkin.Step` struct to execute","ref":"Cucumber.html#apply_step/3-parameters"},{"type":"function","title":"Returns - Cucumber.apply_step/3","doc":"Returns the updated context map if the step succeeds, or raises a `Cucumber.StepError`\nif the step fails or no matching step definition is found.","ref":"Cucumber.html#apply_step/3-returns"},{"type":"macro","title":"Cucumber.defstep/3","doc":"Defines a step pattern and its implementation.\n\nThe `defstep/3` macro is used to define step implementations that match steps in feature files.\nIt supports pattern parameters like `{string}`, `{int}`, `{float}`, and `{word}`.","ref":"Cucumber.html#defstep/3"},{"type":"macro","title":"Parameters - Cucumber.defstep/3","doc":"- `pattern` - The step pattern to match (e.g., \"I click {string} button\")\n- `context` - The variable name to bind the context to (optional)\n- `do` - The block of code to execute when the step matches","ref":"Cucumber.html#defstep/3-parameters"},{"type":"macro","title":"Return Values - Cucumber.defstep/3","doc":"Step implementations can return values in several ways:\n\n- `:ok` - For steps that perform actions but don't need to update context\n- A map - To directly replace the context\n- `{:ok, map}` - To merge new values into the context\n- `{:error, reason}` - To indicate a step failure with a reason","ref":"Cucumber.html#defstep/3-return-values"},{"type":"macro","title":"Examples - Cucumber.defstep/3","doc":"# Simple step with no parameters\n    defstep \"I am on the login page\" do\n      # Setup logic\n      %{page: :login}\n    end\n\n    # Step with string parameter\n    defstep \"I enter {string} in the username field\", context do\n      username = List.first(context.args)\n      {:ok, %{username: username}}\n    end\n\n    # Step with docstring\n    defstep \"I submit the following comment:\", context do\n      # Access the docstring\n      comment_text = context.docstring\n      {:ok, %{comment: comment_text}}\n    end","ref":"Cucumber.html#defstep/3-examples"},{"type":"module","title":"Cucumber.Expression","doc":"Parser and matcher for Cucumber Expressions used in step definitions.\n\nCucumber Expressions are a human-friendly alternative to regular expressions,\nallowing you to define step patterns with typed parameters. This module handles\ncompiling these expressions to regular expressions and extracting/converting parameters.","ref":"Cucumber.Expression.html"},{"type":"module","title":"Parameter Types - Cucumber.Expression","doc":"The following parameter types are supported:\n\n* `{string}` - Matches quoted strings (\"example\") and converts to string\n* `{int}` - Matches integers (42) and converts to integer\n* `{float}` - Matches floating point numbers (3.14) and converts to float\n* `{word}` - Matches a single word (no whitespace) and converts to string","ref":"Cucumber.Expression.html#module-parameter-types"},{"type":"module","title":"Examples - Cucumber.Expression","doc":"# Matching a step with a string parameter\n    \"I click {string} button\" would match:\n    \"I click \"Submit\" button\"\n\n    # Matching a step with multiple parameters\n    \"I add {int} items to my {word} list\" would match:\n    \"I add 5 items to my shopping list\"","ref":"Cucumber.Expression.html#module-examples"},{"type":"function","title":"Cucumber.Expression.compile/1","doc":"Compiles a Cucumber Expression pattern into a regex and parameter converters.\n\nThis function transforms a human-readable pattern with typed parameters\ninto a regular expression for matching and a list of converter functions\nto transform the captured values to their appropriate types.","ref":"Cucumber.Expression.html#compile/1"},{"type":"function","title":"Parameters - Cucumber.Expression.compile/1","doc":"* `pattern` - A string containing a Cucumber Expression pattern","ref":"Cucumber.Expression.html#compile/1-parameters"},{"type":"function","title":"Returns - Cucumber.Expression.compile/1","doc":"Returns a tuple `{regex, converters}` where:\n* `regex` - A compiled regular expression for matching step text\n* `converters` - A list of functions to convert captured values to their appropriate types","ref":"Cucumber.Expression.html#compile/1-returns"},{"type":"function","title":"Examples - Cucumber.Expression.compile/1","doc":"iex> {regex, converters} = Cucumber.Expression.compile(\"I have {int} items\")\n    iex> Regex.match?(regex, \"I have 42 items\")\n    true","ref":"Cucumber.Expression.html#compile/1-examples"},{"type":"function","title":"Cucumber.Expression.match/2","doc":"Matches a step text against a compiled Cucumber Expression.\n\nThis function attempts to match step text against a compiled Cucumber Expression\nand extracts/converts any parameters if there's a match.","ref":"Cucumber.Expression.html#match/2"},{"type":"function","title":"Parameters - Cucumber.Expression.match/2","doc":"* `text` - The step text to match against the pattern\n* `{regex, converters}` - A compiled Cucumber Expression from `compile/1`","ref":"Cucumber.Expression.html#match/2-parameters"},{"type":"function","title":"Returns - Cucumber.Expression.match/2","doc":"Returns one of:\n* `{:match, args}` - If the text matches, where `args` is a list of converted parameter values\n* `:no_match` - If the text doesn't match the expression","ref":"Cucumber.Expression.html#match/2-returns"},{"type":"function","title":"Examples - Cucumber.Expression.match/2","doc":"iex> compiled = Cucumber.Expression.compile(\"I have {int} items\")\n    iex> Cucumber.Expression.match(\"I have 42 items\", compiled)\n    {:match, [42]}\n    iex> Cucumber.Expression.match(\"I have no items\", compiled)\n    :no_match","ref":"Cucumber.Expression.html#match/2-examples"},{"type":"module","title":"Gherkin.Background","doc":"Represents a Gherkin Background section.\n\nA Background contains steps that are run before each scenario in the feature.\nIt allows you to define common setup steps that apply to all scenarios.","ref":"Gherkin.Background.html"},{"type":"type","title":"Gherkin.Background.t/0","doc":"","ref":"Gherkin.Background.html#t:t/0"},{"type":"module","title":"Gherkin.Feature","doc":"Represents a parsed Gherkin feature file (minimal subset).\n\nA Feature is the top-level element in a Gherkin file, containing a name,\noptional description, optional background, and one or more scenarios.\nIt can also have tags that apply to all scenarios in the feature.","ref":"Gherkin.Feature.html"},{"type":"type","title":"Gherkin.Feature.t/0","doc":"","ref":"Gherkin.Feature.html#t:t/0"},{"type":"module","title":"Gherkin.Parser","doc":"Minimal Gherkin 6 parser (Feature, Background, Scenario, Step).\n\nThis module parses Gherkin feature files into Elixir structs, supporting:\n- Feature with name, description, and tags\n- Background with steps\n- Scenarios with steps and tags\n- Steps with keywords, text, docstrings, and datatables\n\nIt implements a subset of the Gherkin language focused on core BDD concepts.","ref":"Gherkin.Parser.html"},{"type":"function","title":"Gherkin.Parser.parse/1","doc":"Parses a Gherkin feature file from a string into structured data.\n\nThis function takes a string containing Gherkin syntax and parses it into a\nstructured `Gherkin.Feature` struct with its associated components.","ref":"Gherkin.Parser.html#parse/1"},{"type":"function","title":"Parameters - Gherkin.Parser.parse/1","doc":"* `gherkin_string` - A string containing Gherkin syntax","ref":"Gherkin.Parser.html#parse/1-parameters"},{"type":"function","title":"Returns - Gherkin.Parser.parse/1","doc":"Returns a `%Gherkin.Feature{}` struct containing:\n* `name` - The feature name\n* `description` - The feature description\n* `tags` - List of feature-level tags\n* `background` - Background steps (if present)\n* `scenarios` - List of scenarios","ref":"Gherkin.Parser.html#parse/1-returns"},{"type":"function","title":"Examples - Gherkin.Parser.parse/1","doc":"# Parse a string containing Gherkin syntax\n    Gherkin.Parser.parse(\"Feature: Shopping Cart\nScenario: Adding an item\")\n    # Returns %Gherkin.Feature{} struct with parsed data","ref":"Gherkin.Parser.html#parse/1-examples"},{"type":"module","title":"Gherkin.Scenario","doc":"Represents a Gherkin Scenario section.\n\nA Scenario is a concrete example that illustrates a business rule.\nIt consists of a name, a list of steps, and optional tags for filtering.","ref":"Gherkin.Scenario.html"},{"type":"type","title":"Gherkin.Scenario.t/0","doc":"","ref":"Gherkin.Scenario.html#t:t/0"},{"type":"module","title":"Gherkin.Step","doc":"Represents a Gherkin step (Given/When/Then/And/But/*).\n\nA Step is a single action or assertion in a scenario. It consists of:\n- keyword: The step type (Given, When, Then, And, But, or *)\n- text: The step text that matches step definitions\n- docstring: Optional multi-line text block (triple-quoted)\n- datatable: Optional table data (pipe-delimited)\n- line: Line number in the source file","ref":"Gherkin.Step.html"},{"type":"type","title":"Gherkin.Step.t/0","doc":"","ref":"Gherkin.Step.html#t:t/0"},{"type":"exception","title":"Cucumber.StepError","doc":"Exception raised when a step in a Cucumber scenario fails.\n\nThis module provides detailed error reporting for two main failure cases:\n\n1. A step has no matching step definition\n2. A step's implementation raises an error or returns `{:error, reason}`\n\nThe error messages include helpful context like:\n- The failing step text and location\n- The scenario name and feature file\n- A history of previously executed steps\n- Suggestions for implementing missing steps","ref":"Cucumber.StepError.html"},{"type":"function","title":"Cucumber.StepError.failed_step/6","doc":"Creates a new step error for a step execution failure.\n\nWhen a step implementation raises an exception or returns `{:error, reason}`,\nthis function creates a detailed error message with context about the failure.","ref":"Cucumber.StepError.html#failed_step/6"},{"type":"function","title":"Parameters - Cucumber.StepError.failed_step/6","doc":"* `step` - The `Gherkin.Step` struct that failed during execution\n* `pattern` - The step pattern that matched the step\n* `failure_reason` - The exception or error reason\n* `feature_file` - The path to the feature file containing the step\n* `scenario_name` - The name of the scenario containing the step\n* `step_history` - A list of previously executed steps with their status","ref":"Cucumber.StepError.html#failed_step/6-parameters"},{"type":"function","title":"Returns - Cucumber.StepError.failed_step/6","doc":"Returns a `Cucumber.StepError` struct with a formatted error message and context.","ref":"Cucumber.StepError.html#failed_step/6-returns"},{"type":"function","title":"Examples - Cucumber.StepError.failed_step/6","doc":"The error message will include details like:\n\n    Step failed:\n\n      Then the result should be 10\n\n    in scenario \"Basic math\" (test/features/calculator.feature:12)\n    matching pattern: \"the result should be {int}\"\n\n    Expected 10 but got 9\n\n    Step execution history:\n      [passed] Given I have entered 5 into the calculator\n      [passed] And I have entered 4 into the calculator\n      [passed] When I press add\n      [failed] Then the result should be 10","ref":"Cucumber.StepError.html#failed_step/6-examples"},{"type":"function","title":"Cucumber.StepError.missing_step_definition/4","doc":"Creates a new step error for a missing step definition.\n\nWhen a step in a feature file doesn't match any step definition, this function\ncreates a helpful error message that includes a suggestion for implementing\nthe missing step.","ref":"Cucumber.StepError.html#missing_step_definition/4"},{"type":"function","title":"Parameters - Cucumber.StepError.missing_step_definition/4","doc":"* `step` - The `Gherkin.Step` struct that has no matching definition\n* `feature_file` - The path to the feature file containing the step\n* `scenario_name` - The name of the scenario containing the step\n* `step_history` - A list of previously executed steps with their status","ref":"Cucumber.StepError.html#missing_step_definition/4-parameters"},{"type":"function","title":"Returns - Cucumber.StepError.missing_step_definition/4","doc":"Returns a `Cucumber.StepError` struct with a formatted error message and context.","ref":"Cucumber.StepError.html#missing_step_definition/4-returns"},{"type":"function","title":"Examples - Cucumber.StepError.missing_step_definition/4","doc":"The error message will include a suggested implementation like:\n\n    defstep \"I click the submit button\", context do\n      # Your step implementation here\n      context\n    end","ref":"Cucumber.StepError.html#missing_step_definition/4-examples"},{"type":"type","title":"Cucumber.StepError.t/0","doc":"","ref":"Cucumber.StepError.html#t:t/0"},{"type":"extras","title":"Getting Started with Cucumber for Elixir","doc":"# Getting Started with Cucumber for Elixir\n\nCucumber for Elixir allows you to write executable specifications using natural language that bridges the gap between technical and non-technical stakeholders.","ref":"getting_started.html"},{"type":"extras","title":"Overview - Getting Started with Cucumber for Elixir","doc":"The Cucumber framework allows you to write tests in a natural language format using Gherkin syntax. This approach enables:\n\n- Behavior-driven development (BDD)\n- Clear documentation of application behavior\n- Tests that serve as living documentation","ref":"getting_started.html#overview"},{"type":"extras","title":"Installation - Getting Started with Cucumber for Elixir","doc":"Add `cucumber` to your `mix.exs` dependencies:\n\n```elixir\ndef deps do\n  [\n    {:cucumber, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"getting_started.html#installation"},{"type":"extras","title":"Quick Start - Getting Started with Cucumber for Elixir","doc":"","ref":"getting_started.html#quick-start"},{"type":"extras","title":"1. Create a Feature File - Getting Started with Cucumber for Elixir","doc":"Feature files use the Gherkin syntax and should be placed in `test/features/` with a `.feature` extension.\n\n```gherkin\n# test/features/user_authentication.feature\nFeature: User Authentication\n\nBackground:\n  Given the application is running\n\nScenario: User signs in with valid credentials\n  Given I am on the sign in page\n  When I enter \"user@example.com\" as my email\n  And I enter \"password123\" as my password\n  And I click the \"Sign In\" button\n  Then I should be redirected to the dashboard\n  And I should see \"Welcome back\" message\n```","ref":"getting_started.html#1-create-a-feature-file"},{"type":"extras","title":"2. Create a Test Module - Getting Started with Cucumber for Elixir","doc":"Create a test module that uses the `Cucumber` macro:\n\n```elixir\ndefmodule UserAuthenticationTest do\n  use Cucumber, feature: \"user_authentication.feature\"\n  \n  # Step definitions\n  defstep \"the application is running\" do\n    # Setup code here\n    :ok\n  end\n  \n  defstep \"I am on the sign in page\", context do\n    # Navigate to sign in page\n    Map.put(context, :current_page, :sign_in)\n  end\n  \n  defstep \"I enter {string} as my email\", context do\n    email = List.first(context.args)\n    # Code to enter email\n    Map.put(context, :email, email)\n  end\n\n  defstep \"I enter {string} as my password\", context do\n    password = List.first(context.args)\n    # Code to enter password\n    Map.put(context, :password, password)\n  end\n  \n  defstep \"I click the {string} button\", context do\n    button_text = List.first(context.args)\n    # Code to click button\n    {:ok, %{clicked: button_text}}\n  end\n  \n  defstep \"I should be redirected to the dashboard\", context do\n    # Assertions for redirection\n    assert context.current_page == :dashboard\n    :ok\n  end\n  \n  defstep \"I should see {string} message\", context do\n    message = List.first(context.args)\n    # Assertion for message\n    assert_text(message)\n    :ok\n  end\nend\n```","ref":"getting_started.html#2-create-a-test-module"},{"type":"extras","title":"3. Run Your Tests - Getting Started with Cucumber for Elixir","doc":"Run your tests using the standard mix test command:\n\n```\nmix test test/lib/user_authentication_test.exs\n```","ref":"getting_started.html#3-run-your-tests"},{"type":"extras","title":"Next Steps - Getting Started with Cucumber for Elixir","doc":"For more detailed information about the Cucumber for Elixir framework, check out these guides:\n\n- [Feature Files](./feature_files.md) - Learn how to write feature files with Gherkin syntax\n- [Step Definitions](./step_definitions.md) - Learn how to implement step definitions\n- [Error Handling](./error_handling.md) - Understanding error reporting and debugging\n- [Best Practices](./best_practices.md) - Best practices and examples","ref":"getting_started.html#next-steps"},{"type":"extras","title":"Writing Feature Files","doc":"# Writing Feature Files\n\nFeature files are the heart of Cucumber testing. They're written in Gherkin syntax, a business-readable domain-specific language that lets you describe application behavior without detailing how that behavior is implemented.","ref":"feature_files.html"},{"type":"extras","title":"Feature File Structure - Writing Feature Files","doc":"Feature files consist of several components:","ref":"feature_files.html#feature-file-structure"},{"type":"extras","title":"Feature - Writing Feature Files","doc":"Every feature file starts with the `Feature:` keyword followed by a name and optional description:\n\n```gherkin\nFeature: Shopping Cart\n  As a user\n  I want to add items to my cart\n  So that I can purchase them later\n```","ref":"feature_files.html#feature"},{"type":"extras","title":"Background - Writing Feature Files","doc":"The `Background:` section contains steps that are executed before each scenario:\n\n```gherkin\nBackground:\n  Given I am logged in as a customer\n  And the product catalog is available\n```","ref":"feature_files.html#background"},{"type":"extras","title":"Scenarios - Writing Feature Files","doc":"Scenarios are concrete examples of how the feature should behave:\n\n```gherkin\nScenario: Adding an item to an empty cart\n  Given I am on the product page for \"Ergonomic Keyboard\"\n  When I click \"Add to Cart\"\n  Then I should see \"Item added to cart\" message\n  And my cart should contain 1 item\n```","ref":"feature_files.html#scenarios"},{"type":"extras","title":"Steps - Writing Feature Files","doc":"Steps use keywords like `Given`, `When`, `Then`, `And`, and `But`:\n\n- `Given`: Establishes preconditions\n- `When`: Describes actions\n- `Then`: Specifies expected outcomes\n- `And`/`But`: Continues the previous step type","ref":"feature_files.html#steps"},{"type":"extras","title":"Step Arguments - Writing Feature Files","doc":"Gherkin supports several types of step arguments:","ref":"feature_files.html#step-arguments"},{"type":"extras","title":"Data Tables - Writing Feature Files","doc":"```gherkin\nScenario: Adding multiple items to cart\n  Given I have the following items in my cart:\n    | Product Name    | Quantity | Price |\n    | Smartphone      | 1        | 699.99|\n    | Protection Plan | 1        | 79.99 |\n  When I proceed to checkout\n  Then the total should be 779.98\n```","ref":"feature_files.html#data-tables"},{"type":"extras","title":"Doc Strings - Writing Feature Files","doc":"```gherkin\nScenario: Submit feedback\n  When I submit the following feedback:\n    \"\"\"\n    I really like your product, but I think\n    it could be improved by adding more features.\n    Keep up the good work!\n    \"\"\"\n  Then my feedback should be recorded\n```","ref":"feature_files.html#doc-strings"},{"type":"extras","title":"File Organization - Writing Feature Files","doc":"Feature files should be placed in a `test/features/` directory and have a `.feature` extension. Organize them logically by feature or domain area:\n\n```\ntest/\n└── features/\n    ├── authentication/\n    │   ├── login.feature\n    │   └── registration.feature\n    ├── shopping/\n    │   ├── cart.feature\n    │   └── checkout.feature\n    └── user_profile.feature\n```","ref":"feature_files.html#file-organization"},{"type":"extras","title":"Tags - Writing Feature Files","doc":"Tags are used to categorize scenarios and can be used for filtering which tests to run:\n\n```gherkin\n@authentication @important\nFeature: User Login\n\n@happy_path\nScenario: Successful login with valid credentials\n  Given I am on the login page\n  When I enter valid credentials\n  Then I should be logged in\n\n@error_handling\nScenario: Failed login with invalid credentials\n  Given I am on the login page\n  When I enter invalid credentials\n  Then I should see an error message\n```\n\nTags can be used at the Feature level (applies to all scenarios) or at the Scenario level.","ref":"feature_files.html#tags"},{"type":"extras","title":"Best Practices for Feature Files - Writing Feature Files","doc":"1. **Keep language simple and consistent** - Use clear, business-focused terminology\n2. **One scenario per behavior** - Each scenario should test one specific behavior\n3. **Be specific in examples** - Use concrete values rather than vague descriptions\n4. **Use background wisely** - Only for steps that are truly common to all scenarios\n5. **Limit scenario length** - If a scenario has many steps, consider breaking it down\n6. **Organize with tags** - Use tags to categorize and group related scenarios\n7. **Think like a user** - Write scenarios from the user's perspective\n8. **Don't get too technical** - Avoid technical implementation details in Gherkin","ref":"feature_files.html#best-practices-for-feature-files"},{"type":"extras","title":"Step Definitions","doc":"# Step Definitions\n\nStep definitions connect the Gherkin steps in your feature files to actual code. They're the glue between your natural language specifications and the implementation that tests your application.","ref":"step_definitions.html"},{"type":"extras","title":"Basic Step Definition - Step Definitions","doc":"Step definitions are created using the `defstep` macro:\n\n```elixir\ndefstep \"I am logged in as a customer\", context do\n  # Authentication logic here\n  {:ok, %{user: create_and_login_customer()}}\nend\n```","ref":"step_definitions.html#basic-step-definition"},{"type":"extras","title":"Steps with Parameters - Step Definitions","doc":"Cucumber supports several parameter types that can be used in step patterns:","ref":"step_definitions.html#steps-with-parameters"},{"type":"extras","title":"String Parameters - Step Definitions","doc":"```elixir\ndefstep \"I am on the product page for {string}\", context do\n  product_name = List.first(context.args)\n  # Navigate to product page\n  {:ok, %{current_page: :product, product_name: product_name}}\nend\n```","ref":"step_definitions.html#string-parameters"},{"type":"extras","title":"Integer Parameters - Step Definitions","doc":"```elixir\ndefstep \"I should have {int} items in my wishlist\", context do\n  expected_count = List.first(context.args)\n  # Assertion for wishlist count\n  assert get_wishlist_count() == expected_count\n  :ok\nend\n```","ref":"step_definitions.html#integer-parameters"},{"type":"extras","title":"Float Parameters - Step Definitions","doc":"```elixir\ndefstep \"the total price should be {float}\", context do\n  expected_total = List.first(context.args)\n  # Assertion for price\n  assert_in_delta get_cart_total(), expected_total, 0.01\n  :ok\nend\n```","ref":"step_definitions.html#float-parameters"},{"type":"extras","title":"Word Parameters - Step Definitions","doc":"```elixir\ndefstep \"I should see the {word} dashboard\", context do\n  dashboard_type = List.first(context.args)\n  # Assertion for dashboard type\n  assert get_current_dashboard() == dashboard_type\n  :ok\nend\n```","ref":"step_definitions.html#word-parameters"},{"type":"extras","title":"Working with Data Tables - Step Definitions","doc":"In your feature file:\n```gherkin\nGiven I have the following items in my cart:\n  | Product Name    | Quantity | Price |\n  | Smartphone      | 1        | 699.99|\n  | Protection Plan | 1        | 79.99 |\n```\n\nIn your test module:\n```elixir\ndefstep \"I have the following items in my cart:\", context do\n  # Access the datatable\n  datatable = context.datatable\n\n  # Access headers\n  headers = datatable.headers  # [\"Product Name\", \"Quantity\", \"Price\"]\n  \n  # Access rows as maps\n  items = datatable.maps\n  # [\n  #   %{\"Product Name\" => \"Smartphone\", \"Quantity\" => \"1\", \"Price\" => \"699.99\"},\n  #   %{\"Product Name\" => \"Protection Plan\", \"Quantity\" => \"1\", \"Price\" => \"79.99\"}\n  # ]\n  \n  # Process the items\n  {:ok, %{cart_items: items}}\nend\n```","ref":"step_definitions.html#working-with-data-tables"},{"type":"extras","title":"Working with Doc Strings - Step Definitions","doc":"In your feature file:\n```gherkin\nWhen I submit the following feedback:\n  \"\"\"\n  I really like your product, but I think\n  it could be improved by adding more features.\n  Keep up the good work!\n  \"\"\"\n```\n\nIn your test module:\n```elixir\ndefstep \"I submit the following feedback:\", context do\n  feedback_text = context.docstring\n  # Submit feedback logic\n  {:ok, %{submitted_feedback: feedback_text}}\nend\n```","ref":"step_definitions.html#working-with-doc-strings"},{"type":"extras","title":"Return Values - Step Definitions","doc":"Step definitions can return values in several ways:","ref":"step_definitions.html#return-values"},{"type":"extras","title":"1. Return :ok - Step Definitions","doc":"For steps that perform actions but don't need to update context:\n\n```elixir\ndefstep \"I click the submit button\", _context do\n  # Click logic\n  :ok\nend\n```","ref":"step_definitions.html#1-return-ok"},{"type":"extras","title":"2. Return a Map - Step Definitions","doc":"To directly replace the context:\n\n```elixir\ndefstep \"I am on the home page\", _context do\n  %{current_page: :home}\nend\n```","ref":"step_definitions.html#2-return-a-map"},{"type":"extras","title":"3. Return {:ok, map} - Step Definitions","doc":"To merge new values into the context:\n\n```elixir\ndefstep \"I search for {string}\", context do\n  search_term = List.first(context.args)\n  # Search logic\n  {:ok, %{search_term: search_term, search_results: perform_search(search_term)}}\nend\n```","ref":"step_definitions.html#3-return-ok-map"},{"type":"extras","title":"4. Return {:error, reason} - Step Definitions","doc":"To indicate a step failure with a reason:\n\n```elixir\ndefstep \"the payment should be successful\", context do\n  if context.payment_status == :success do\n    :ok\n  else\n    {:error, \"Expected payment to succeed, but got status: #{context.payment_status}\"}\n  end\nend\n```","ref":"step_definitions.html#4-return-error-reason"},{"type":"extras","title":"Step Patterns and Matching - Step Definitions","doc":"When a Gherkin step needs to be executed, the framework searches through all step definitions for a matching pattern. The matching process:\n\n1. Starts with the current module's step definitions\n2. Looks for an exact match first\n3. Then tries to match using parameter placeholders\n4. Raises an error if no matching step definition is found","ref":"step_definitions.html#step-patterns-and-matching"},{"type":"extras","title":"Context Management - Step Definitions","doc":"The context object is a map that flows from step to step during a scenario's execution:\n\n- It starts as an empty map or the ExUnit test context\n- Each step can add to, modify, or replace the context\n- Any values added to the context are available to subsequent steps\n- It's useful for sharing state between steps, such as user sessions, form data, etc.\n\nExample of context flow:\n\n```elixir\ndefstep \"I am on the login page\", _context do\n  # Initial step sets up the page\n  %{page: :login}\nend\n\ndefstep \"I enter my credentials\", context do\n  # Second step uses the page from previous step and adds credentials\n  assert context.page == :login\n  {:ok, %{username: \"testuser\", password: \"password123\"}}\nend\n\ndefstep \"I click the login button\", context do\n  # Third step can access all previous context values\n  assert context.page == :login\n  assert context.username == \"testuser\" \n  assert context.password == \"password123\"\n  \n  # And add more context values\n  {:ok, %{logged_in: true, user_id: 123}}\nend\n```","ref":"step_definitions.html#context-management"},{"type":"extras","title":"Best Practices for Step Definitions - Step Definitions","doc":"1. **Keep steps reusable** - Write generic steps that can be used across features\n2. **One assertion per step** - Especially for \"Then\" steps\n3. **Use context for state management** - Pass necessary data between steps\n4. **Handle errors gracefully** - Provide helpful error messages\n5. **Name steps from the user's perspective** - Focus on what, not how\n6. **Organize steps logically** - Group related steps together\n7. **Document complex steps** - Add comments for steps with complex logic\n8. **Avoid implementation details in step patterns** - Keep to business terminology","ref":"step_definitions.html#best-practices-for-step-definitions"},{"type":"extras","title":"Error Handling and Debugging","doc":"# Error Handling and Debugging\n\nCucumber for Elixir provides detailed error reporting to help debug test failures. This document explains how to handle and understand errors in your Cucumber tests.","ref":"error_handling.html"},{"type":"extras","title":"Types of Errors - Error Handling and Debugging","doc":"","ref":"error_handling.html#types-of-errors"},{"type":"extras","title":"Missing Step Definition - Error Handling and Debugging","doc":"When a step in your feature file has no matching definition, the framework will provide a helpful error message:\n\n```\n** (Cucumber.StepError) No matching step definition found for step:\n\n  When I try to use a step with no definition\n\nin scenario \"Missing Step Example\" (test/features/example.feature:6)\n\nPlease define this step with:\n\ndefstep \"I try to use a step with no definition\", context do\n  # Your step implementation here\n  context\nend\n```\n\nThis error not only tells you which step is missing, but also provides a template for implementing the missing step definition.","ref":"error_handling.html#missing-step-definition"},{"type":"extras","title":"Failed Step - Error Handling and Debugging","doc":"When a step fails during execution (due to a failed assertion or an exception), you'll get an error like this:\n\n```\n** (Cucumber.StepError) Step failed:\n\n  Then the validation should succeed\n\nin scenario \"Form Submission\" (test/features/forms.feature:12)\nmatching pattern: \"the validation should succeed\"\n\nValidation failed: invalid input data\n\nStep execution history:\n  [passed] Given a form to fill out\n  [passed] When I submit invalid data\n  [failed] Then the validation should succeed\n```\n\nThe error message includes:\n- Which step failed\n- The scenario and file where the failure occurred\n- The step pattern that matched\n- The error message from the step implementation\n- The execution history, showing which steps passed and which failed","ref":"error_handling.html#failed-step"},{"type":"extras","title":"Syntax Errors in Feature Files - Error Handling and Debugging","doc":"If there are syntax errors in your feature files, you'll get an error when the parser tries to process the file:\n\n```\n** (Cucumber.ParseError) Syntax error in feature file:\n\n  test/features/invalid.feature:5\n\nExpected a scenario or background but found:\n  \n  Invalid line that doesn't start with a Gherkin keyword\n```","ref":"error_handling.html#syntax-errors-in-feature-files"},{"type":"extras","title":"Step Failure Handling - Error Handling and Debugging","doc":"Step definitions can indicate failure in several ways:","ref":"error_handling.html#step-failure-handling"},{"type":"extras","title":"1. Assertions - Error Handling and Debugging","doc":"Using ExUnit assertions will cause the step to fail if the assertion fails:\n\n```elixir\ndefstep \"the total should be {float}\", context do\n  total = List.first(context.args)\n  assert context.cart_total == total\n  :ok\nend\n```","ref":"error_handling.html#1-assertions"},{"type":"extras","title":"2. Raising Exceptions - Error Handling and Debugging","doc":"Any uncaught exception will cause the step to fail:\n\n```elixir\ndefstep \"I click the submit button\", _context do\n  raise \"The submit button is disabled\"\n  :ok\nend\n```","ref":"error_handling.html#2-raising-exceptions"},{"type":"extras","title":"3. Returning {:error, reason} - Error Handling and Debugging","doc":"You can explicitly return an error tuple to fail a step:\n\n```elixir\ndefstep \"the payment should be successful\", context do\n  if context.payment_status == :success do\n    :ok\n  else\n    {:error, \"Expected payment to succeed, but got status: #{context.payment_status}\"}\n  end\nend\n```","ref":"error_handling.html#3-returning-error-reason"},{"type":"extras","title":"Debugging Tips - Error Handling and Debugging","doc":"","ref":"error_handling.html#debugging-tips"},{"type":"extras","title":"1. Use IO.inspect for Debug Output - Error Handling and Debugging","doc":"Insert `IO.inspect` calls to see the values of variables during test execution:\n\n```elixir\ndefstep \"I should see my order summary\", context do\n  IO.inspect(context, label: \"Context in order summary step\")\n  assert context.order != nil\n  :ok\nend\n```","ref":"error_handling.html#1-use-io-inspect-for-debug-output"},{"type":"extras","title":"2. Add Step Execution Logs - Error Handling and Debugging","doc":"Log information about step execution:\n\n```elixir\ndefstep \"I complete the checkout process\", context do\n  IO.puts(\"Starting checkout process\")\n  # Checkout logic\n  IO.puts(\"Completed checkout process\")\n  :ok\nend\n```","ref":"error_handling.html#2-add-step-execution-logs"},{"type":"extras","title":"3. Examine the Full Context - Error Handling and Debugging","doc":"Print the full context at any point to see the accumulated state:\n\n```elixir\ndefstep \"I check my context\", context do\n  IO.inspect(context, label: \"Current context\", pretty: true)\n  :ok\nend\n```","ref":"error_handling.html#3-examine-the-full-context"},{"type":"extras","title":"4. Create Debug-Only Steps - Error Handling and Debugging","doc":"Add steps specifically for debugging:\n\n```elixir\ndefstep \"I debug my test state\", context do\n  IO.puts(\"==== DEBUG STATE ====\")\n  IO.inspect(context.current_page, label: \"Current Page\")\n  IO.inspect(context.user, label: \"Current User\")\n  IO.inspect(context.cart_items, label: \"Cart Items\")\n  IO.puts(\"==== END DEBUG ====\")\n  :ok\nend\n```","ref":"error_handling.html#4-create-debug-only-steps"},{"type":"extras","title":"Handling Flaky Tests - Error Handling and Debugging","doc":"Sometimes tests can be inconsistent due to timing issues, especially with UI interactions:\n\n```elixir\ndefstep \"I should see the confirmation message\", context do\n  # Add delay or retry logic for UI-related assertions\n  :timer.sleep(500)  # Give the UI time to update\n  assert_text(\"Your order has been confirmed\")\n  :ok\nend\n```\n\nConsider implementing retry logic for flaky steps:\n\n```elixir\ndefp retry_until(function, max_attempts \\\\ 5, delay \\\\ 100) do\n  Enum.reduce_while(1..max_attempts, nil, fn attempt, _acc ->\n    case function.() do\n      {:ok, result} -> {:halt, {:ok, result}}\n      {:error, reason} ->\n        if attempt == max_attempts do\n          {:halt, {:error, reason}}\n        else\n          :timer.sleep(delay)\n          {:cont, nil}\n        end\n    end\n  end)\nend\n\ndefstep \"I should see the success notification\", _context do\n  result = retry_until(fn ->\n    if element_visible?(\".success-notification\") do\n      {:ok, true}\n    else\n      {:error, \"Success notification not visible\"}\n    end\n  end)\n  \n  case result do\n    {:ok, _} -> :ok\n    {:error, reason} -> {:error, reason}\n  end\nend\n```","ref":"error_handling.html#handling-flaky-tests"},{"type":"extras","title":"Common Error Patterns and Solutions - Error Handling and Debugging","doc":"| Error Pattern | Possible Cause | Solution |\n|---------------|----------------|----------|\n| Step not found | Step definition missing or typo in step | Create the missing step definition or correct the typo |\n| Assertion failure | Expected value doesn't match actual | Check your test data and application logic |\n| Timeout | Asynchronous operation didn't complete in time | Increase timeout or add retry logic |\n| Element not found | UI element not rendered yet or selector wrong | Add delay, retry, or correct the selector |\n| Context key missing | Previous step didn't set expected data | Ensure required context keys are set in earlier steps |","ref":"error_handling.html#common-error-patterns-and-solutions"},{"type":"extras","title":"Best Practices for Cucumber Tests","doc":"# Best Practices for Cucumber Tests\n\nThis guide outlines best practices for writing and organizing your Cucumber tests to ensure they remain maintainable, readable, and effective.","ref":"best_practices.html"},{"type":"extras","title":"Feature File Organization - Best Practices for Cucumber Tests","doc":"","ref":"best_practices.html#feature-file-organization"},{"type":"extras","title":"Directory Structure - Best Practices for Cucumber Tests","doc":"```\ntest/\n├── features/           # Feature files\n│   ├── authentication/ # Feature grouping by domain\n│   │   ├── login.feature\n│   │   └── registration.feature\n│   └── shopping/       # Another domain\n│       ├── cart.feature\n│       └── checkout.feature\n└── lib/                # Test modules with step definitions\n    ├── authentication_test.exs\n    └── shopping_test.exs\n```","ref":"best_practices.html#directory-structure"},{"type":"extras","title":"Naming Conventions - Best Practices for Cucumber Tests","doc":"- Use snake_case for feature file names\n- Group related features into subdirectories\n- Name test modules with a descriptive suffix (e.g., `LoginTest`, `CheckoutTest`)","ref":"best_practices.html#naming-conventions"},{"type":"extras","title":"Writing Good Scenarios - Best Practices for Cucumber Tests","doc":"","ref":"best_practices.html#writing-good-scenarios"},{"type":"extras","title":"Scenario Best Practices - Best Practices for Cucumber Tests","doc":"1. **Keep scenarios focused**: Each scenario should test one specific behavior\n2. **Be consistent**: Use consistent language across scenarios\n3. **Use concrete examples**: Prefer specific, realistic values to abstract placeholders\n4. **Avoid technical details**: Keep scenarios in business language\n5. **Keep them short**: Aim for 3-7 steps per scenario\n6. **Use backgrounds wisely**: Only for truly common setup steps","ref":"best_practices.html#scenario-best-practices"},{"type":"extras","title":"Example: Bad vs. Good - Best Practices for Cucumber Tests","doc":"Bad:\n```gherkin\nScenario: User interaction\n  Given a user\n  When the user does stuff\n  Then the outcome is good\n```\n\nGood:\n```gherkin\nScenario: Customer adds product to cart from product detail page\n  Given I am logged in as \"john@example.com\"\n  And I am viewing the product \"Ergonomic Keyboard\"\n  When I click the \"Add to Cart\" button\n  Then I should see \"Product added to cart\" message\n  And my cart should contain 1 item\n```","ref":"best_practices.html#example-bad-vs-good"},{"type":"extras","title":"Step Definition Best Practices - Best Practices for Cucumber Tests","doc":"","ref":"best_practices.html#step-definition-best-practices"},{"type":"extras","title":"Organization - Best Practices for Cucumber Tests","doc":"1. **Group related step definitions**: Keep related steps together in the same file\n2. **Use helper functions**: Extract common functionality into helper functions\n3. **Create reusable steps**: Design steps to be reused across scenarios","ref":"best_practices.html#organization"},{"type":"extras","title":"Naming Steps - Best Practices for Cucumber Tests","doc":"1. **Use the actor's perspective**: \"I click the button\" rather than \"Button is clicked\"\n2. **Be specific**: \"I submit the registration form\" rather than \"I submit the form\"\n3. **Avoid technical implementation details**: \"I click Login\" rather than \"I click #login-button\"","ref":"best_practices.html#naming-steps"},{"type":"extras","title":"Step Implementation - Best Practices for Cucumber Tests","doc":"```elixir\n# Good: Clear, focused, with good error messages\ndefstep \"I click the {string} button\", context do\n  button_text = List.first(context.args)\n  \n  case find_button(button_text) do\n    {:ok, button} -> \n      click(button)\n      :ok\n    {:error, :not_found} ->\n      {:error, \"Button with text '#{button_text}' not found on page\"}\n  end\nend\n\n# Bad: Vague, doing too much, poor error handling\ndefstep \"I interact with the page\", _context do\n  find_element(\"#some-button\") |> click()\n  :ok\nend\n```","ref":"best_practices.html#step-implementation"},{"type":"extras","title":"Context Management - Best Practices for Cucumber Tests","doc":"","ref":"best_practices.html#context-management"},{"type":"extras","title":"Passing Data Between Steps - Best Practices for Cucumber Tests","doc":"```elixir\n# First step establishes context\ndefstep \"I submit an order for {string}\", _context do\n  product_name = List.first(context.args)\n  order_id = create_order(product_name)\n  \n  # Return context to be used in following steps\n  {:ok, %{product_name: product_name, order_id: order_id}}\nend\n\n# Second step uses that context\ndefstep \"I should receive an order confirmation email\", context do\n  # Access data from previous step\n  assert_email_sent(\n    to: context.user.email,\n    subject: \"Order Confirmation for ##{context.order_id}\",\n    containing: context.product_name\n  )\n  \n  :ok\nend\n```","ref":"best_practices.html#passing-data-between-steps"},{"type":"extras","title":"Context Guidelines - Best Practices for Cucumber Tests","doc":"1. **Be explicit**: Only store what's needed for subsequent steps\n2. **Consider naming**: Use descriptive keys in the context map\n3. **Clean up after yourself**: Don't let the context grow too large","ref":"best_practices.html#context-guidelines"},{"type":"extras","title":"Testing Different Layers - Best Practices for Cucumber Tests","doc":"","ref":"best_practices.html#testing-different-layers"},{"type":"extras","title":"UI Testing - Best Practices for Cucumber Tests","doc":"```elixir\ndefstep \"I click the {string} button\", context do\n  button_text = List.first(context.args)\n  click_button(button_text)\n  :ok\nend\n```","ref":"best_practices.html#ui-testing"},{"type":"extras","title":"API Testing - Best Practices for Cucumber Tests","doc":"```elixir\ndefstep \"I make a GET request to {string}\", context do\n  endpoint = List.first(context.args)\n  response = HTTPoison.get!(\"#{context.base_url}#{endpoint}\")\n  {:ok, Map.put(context, :response, response)}\nend\n\ndefstep \"the response status should be {int}\", context do\n  expected_status = List.first(context.args)\n  assert context.response.status_code == expected_status\n  :ok\nend\n```","ref":"best_practices.html#api-testing"},{"type":"extras","title":"Database Testing - Best Practices for Cucumber Tests","doc":"```elixir\ndefstep \"there should be a user in the database with email {string}\", context do\n  email = List.first(context.args)\n  user = Repo.get_by(User, email: email)\n  assert user != nil\n  :ok\nend\n```","ref":"best_practices.html#database-testing"},{"type":"extras","title":"Handling Test Data - Best Practices for Cucumber Tests","doc":"","ref":"best_practices.html#handling-test-data"},{"type":"extras","title":"Using Factory Functions - Best Practices for Cucumber Tests","doc":"```elixir\n# Helper function to create test data\ndefp create_test_user(attrs \\\\ %{}) do\n  defaults = %{\n    username: \"testuser\",\n    email: \"test@example.com\",\n    password: \"password123\"\n  }\n  \n  Map.merge(defaults, attrs)\n  |> User.changeset()\n  |> Repo.insert!()\nend\n\n# Use in step definitions\ndefstep \"a user exists with email {string}\", context do\n  email = List.first(context.args)\n  user = create_test_user(%{email: email})\n  {:ok, %{user: user}}\nend\n```","ref":"best_practices.html#using-factory-functions"},{"type":"extras","title":"Using Tags for Test Environment - Best Practices for Cucumber Tests","doc":"```gherkin\n@require_db_cleanup\nFeature: User Management\n\nScenario: Create a new user\n  When I create a user with email \"newuser@example.com\"\n  Then the user should exist in the database\n```\n\n```elixir\nsetup context do\n  if \"require_db_cleanup\" in context.feature_tags do\n    on_exit(fn -> \n      # Clean up database after tests\n      Repo.delete_all(User)\n    end)\n  end\n  \n  context\nend\n```","ref":"best_practices.html#using-tags-for-test-environment"},{"type":"extras","title":"Common Testing Patterns - Best Practices for Cucumber Tests","doc":"","ref":"best_practices.html#common-testing-patterns"},{"type":"extras","title":"The Given-When-Then Formula - Best Practices for Cucumber Tests","doc":"1. **Given**: Establishes the initial context\n2. **When**: Describes the key action\n3. **Then**: Specifies expected outcomes","ref":"best_practices.html#the-given-when-then-formula"},{"type":"extras","title":"Table-Driven Scenarios - Best Practices for Cucumber Tests","doc":"```gherkin\nScenario Outline: User registration with different passwords\n  When I try to register with email \"user@example.com\" and password \" \"\n  Then registration should be \" \"\n  And I should see message \" \"\n  \n  Examples:\n    | password      | result    | message                            |\n    | pass          | failed    | Password is too short              |\n    | password123   | success   | Registration successful            |\n    | noUppercase1  | failed    | Password needs an uppercase letter |\n    | NoNumbers     | failed    | Password needs a number            |\n```","ref":"best_practices.html#table-driven-scenarios"},{"type":"extras","title":"State-Based Testing - Best Practices for Cucumber Tests","doc":"```gherkin\nScenario: Completed order cannot be modified\n  Given I have an order with id \"12345\"\n  And the order status is \"completed\"\n  When I attempt to modify the order\n  Then I should receive an error \"Cannot modify completed order\"\n```","ref":"best_practices.html#state-based-testing"},{"type":"extras","title":"Continuous Integration - Best Practices for Cucumber Tests","doc":"","ref":"best_practices.html#continuous-integration"},{"type":"extras","title":"Running Tests in CI - Best Practices for Cucumber Tests","doc":"```yaml\n# Example GitHub Actions workflow\njobs:\n  test:\n    runs-on: ubuntu-latest\n    steps:\n      - uses: actions/checkout@v2\n      - uses: erlef/setup-beam@v1\n        with:\n          otp-version: '25'\n          elixir-version: '1.14'\n      - run: mix deps.get\n      - run: mix test\n```","ref":"best_practices.html#running-tests-in-ci"},{"type":"extras","title":"Reporting Test Results - Best Practices for Cucumber Tests","doc":"Consider tools for generating test reports:\n\n```elixir\n# In test_helper.exs\nExUnit.configure(\n  formatters: [ExUnit.CLIFormatter, JUnitFormatter]\n)\n```","ref":"best_practices.html#reporting-test-results"},{"type":"extras","title":"Advanced Techniques - Best Practices for Cucumber Tests","doc":"","ref":"best_practices.html#advanced-techniques"},{"type":"extras","title":"Custom Parameter Types - Best Practices for Cucumber Tests","doc":"```elixir\n# Custom parameter type for dates\ndefmodule DateParameterType do\n  def match(\"today\") do\n    Date.utc_today()\n  end\n  \n  def match(\"tomorrow\") do\n    Date.add(Date.utc_today(), 1)\n  end\n  \n  def match(date_string) do\n    case Date.from_iso8601(date_string) do\n      {:ok, date} -> date\n      _ -> raise \"Invalid date format: #{date_string}. Use ISO-8601 format.\"\n    end\n  end\nend\n\n# Using in step definitions\ndefstep \"I schedule an appointment for {date}\", context do\n  date = List.first(context.args)\n  # date is already a Date struct\n  {:ok, %{appointment_date: date}}\nend\n```","ref":"best_practices.html#custom-parameter-types"},{"type":"extras","title":"Sharing Steps Between Test Modules - Best Practices for Cucumber Tests","doc":"```elixir\n# In a shared module\ndefmodule CommonSteps do\n  defmacro __using__(_opts) do\n    quote do\n      defstep \"I am logged in as {string}\", context do\n        username = List.first(context.args)\n        # Login logic\n        {:ok, %{current_user: find_user(username)}}\n      end\n      \n      defstep \"I should be on the {string} page\", context do\n        page_name = List.first(context.args)\n        assert current_page() == page_name\n        :ok\n      end\n    end\n  end\nend\n\n# In a test module\ndefmodule AuthenticationTest do\n  use Cucumber, feature: \"authentication.feature\"\n  use CommonSteps\n  \n  # Additional step definitions specific to this module\nend\n```","ref":"best_practices.html#sharing-steps-between-test-modules"},{"type":"extras","title":"Performance Considerations - Best Practices for Cucumber Tests","doc":"1. **Minimize external dependencies**: Mock third-party services\n2. **Optimize database usage**: Use transactions, clean up test data\n3. **Reuse browser sessions**: For UI tests, avoid creating new sessions for each scenario\n4. **Parallelization**: Consider running tests in parallel when possible","ref":"best_practices.html#performance-considerations"},{"type":"extras","title":"Cucumber Architecture","doc":"# Cucumber Architecture\n\nThis document provides an overview of the Cucumber implementation architecture, explaining the core components and how they interact.","ref":"architecture.html"},{"type":"extras","title":"Core Components - Cucumber Architecture","doc":"```\nCucumber\n  ├── Gherkin (Parser)\n  ├── Expression (Parameter Matching)\n  ├── Runner (Test Execution)\n  └── Formatter (Output)\n```","ref":"architecture.html#core-components"},{"type":"extras","title":"Gherkin Parser - Cucumber Architecture","doc":"The Gherkin parser is responsible for parsing `.feature` files into a structured format that can be executed. It handles the syntax of Gherkin, including:\n\n- Feature declarations\n- Scenario outlines\n- Backgrounds\n- Steps (Given, When, Then)\n- Tables and doc strings\n- Tags\n\nThe parser produces an Abstract Syntax Tree (AST) that represents the structure of the feature file.\n\n```elixir\n# Simplified representation of the Gherkin parser flow\nFeature File (Text) → Lexer → Tokens → Parser → AST\n```","ref":"architecture.html#gherkin-parser"},{"type":"extras","title":"Expression Engine - Cucumber Architecture","doc":"The Expression engine is responsible for matching step text against step definitions. It supports:\n\n- Regular expressions\n- Cucumber expressions (a simplified syntax with parameter types)\n- Parameter conversion (string to typed values)\n\n```elixir\ndefmodule Cucumber.Expression do\n  # Converts a cucumber expression into a regex and parameter converters\n  def compile(pattern) do\n    # Transforms {string}, {int}, etc. into regex patterns\n    # Returns {regex, converters}\n  end\n\n  # Matches text against a compiled expression\n  def match(text, {regex, converters}) do\n    # Returns {:match, args} or :no_match\n  end\nend\n```","ref":"architecture.html#expression-engine"},{"type":"extras","title":"Runner - Cucumber Architecture","doc":"The Runner is responsible for executing the parsed features against step definitions. It:\n\n1. Loads feature files\n2. Finds matching step definitions\n3. Executes steps in order\n4. Manages test context between steps\n5. Handles errors and reporting\n\n```\nFeature → Scenarios → Steps → Step Definitions → Execution\n```","ref":"architecture.html#runner"},{"type":"extras","title":"Context Management - Cucumber Architecture","doc":"The context is a map that's passed between step definitions, allowing them to share state. The runner:\n\n1. Creates an initial context\n2. Passes it to each step\n3. Collects the updated context\n4. Passes the updated context to the next step\n\n```elixir\n# Example context flow\ninitial_context = %{feature_name: \"Authentication\"}\n{:ok, updated_context} = execute_step(step1, initial_context)\n{:ok, final_context} = execute_step(step2, updated_context)\n```","ref":"architecture.html#context-management"},{"type":"extras","title":"Integration with ExUnit - Cucumber Architecture","doc":"The Cucumber library integrates with ExUnit through:\n\n1. Custom ExUnit case modules\n2. Test module generation\n3. Test callbacks (setup, teardown)\n\n```elixir\n# Simplified representation of ExUnit integration\ndefmodule MyTest do\n  use Cucumber, feature: \"my.feature\"\n  \n  # Step definitions become test functions\n  # Scenarios become test cases\nend\n```","ref":"architecture.html#integration-with-exunit"},{"type":"extras","title":"Macro System - Cucumber Architecture","doc":"Cucumber uses Elixir's macro system extensively to provide a clean DSL:\n\n```elixir\ndefmodule MyTest do\n  use Cucumber, feature: \"authentication.feature\"\n  \n  # Macros transform these into functions\n  defstep \"I am on the login page\", _context do\n    # Implementation\n    :ok\n  end\nend\n```\n\nAt compile time, these macros:\n\n1. Read the feature file\n2. Generate ExUnit test cases\n3. Register step definitions\n4. Set up test callbacks","ref":"architecture.html#macro-system"},{"type":"extras","title":"Extension Points - Cucumber Architecture","doc":"The architecture provides several extension points:\n\n1. **Custom Parameter Types**: Extend the Expression engine with new types\n2. **Formatters**: Create custom output formats\n3. **Hooks**: Add before/after hooks at different levels\n4. **Tags**: Filter and customize execution based on tags","ref":"architecture.html#extension-points"},{"type":"extras","title":"Implementation Details - Cucumber Architecture","doc":"","ref":"architecture.html#implementation-details"},{"type":"extras","title":"Optimizations - Cucumber Architecture","doc":"- **Step Definition Registry**: Fast lookup of step definitions\n- **Pattern Compilation**: Expressions are compiled once and reused\n- **Lazy Loading**: Feature files are parsed on demand","ref":"architecture.html#optimizations"},{"type":"extras","title":"Error Handling - Cucumber Architecture","doc":"When a step fails, the system:\n\n1. Captures the error information\n2. Adds context about the feature and scenario\n3. Reports detailed failure information\n4. Stops execution of the current scenario\n5. Continues with the next scenario","ref":"architecture.html#error-handling"},{"type":"extras","title":"Execution Flow - Cucumber Architecture","doc":"```\n1. Feature loading\n   ├── Parse feature files\n   └── Build execution plan\n\n2. Test compilation\n   ├── Generate ExUnit tests\n   └── Register step definitions\n\n3. Test execution\n   ├── Setup test environment\n   ├── Execute steps\n   │   ├── Find matching step definition\n   │   ├── Apply parameter conversions\n   │   ├── Execute step function\n   │   └── Manage context between steps\n   └── Report results\n```","ref":"architecture.html#execution-flow"},{"type":"extras","title":"Code Structure - Cucumber Architecture","doc":"```\nlib/\n├── cucumber.ex             # Main module and API\n├── gherkin.ex              # Feature file parser\n├── cucumber/\n    ├── expression.ex       # Step matching engine\n    ├── runner.ex           # Test execution\n    ├── formatter.ex        # Output formatting\n    ├── step_definition.ex  # Step definition handling\n    └── hooks.ex            # Before/after hooks\n```","ref":"architecture.html#code-structure"},{"type":"extras","title":"Runtime Behavior - Cucumber Architecture","doc":"1. The `use Cucumber` macro transforms the module at compile time\n2. ExUnit runs the generated test cases\n3. Each scenario becomes a test case\n4. Each step invocation:\n   - Finds the matching step definition\n   - Extracts parameters\n   - Runs the step function\n   - Manages the context\n5. Results are reported through ExUnit's reporting system\n\nThis architecture provides a solid foundation that balances simplicity of use with flexibility for extension.","ref":"architecture.html#runtime-behavior"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}